@@ -8,11 +8,11 @@ use strict;
 
 package Date::Set;
 
-use Set::Infinite ':all';
+use Set::Infinite;
 use vars qw(@ISA @EXPORT @EXPORT_OK  $AUTOLOAD $VERSION
   %FREQ %WEEKDAY %WHICH_OCCURRENCE
   $FUTURE $PAST $FOREVER $NEVER
-  $WKST
+  $WKST $inf
   $DEBUG
   );    # perl standard stuff / lookup tables / date sets / debug
 
@@ -20,8 +20,8 @@ use AutoLoader;
 use Carp;
 @ISA       = qw(Set::Infinite);
 @EXPORT    = qw();
-@EXPORT_OK = qw(type);
-$VERSION = (qw'$Revision: 1.23_04 $')[1];
+@EXPORT_OK = qw(type $inf inf);
+$VERSION = (qw'$Revision: 1.24_08 $')[1];
 
 
 #----- initialize package globals
@@ -29,6 +29,8 @@ $VERSION = (qw'$Revision: 1.23_04 $')[1];
 $DEBUG                = 0;
 $Set::Infinite::TRACE = 0;
 Set::Infinite::type('Date::Set::ICal');
+$inf = 10**10**10;
+sub inf { $inf }
 
 $WKST = 'MO';      # by default, weeks start on monday
 
@@ -62,6 +64,7 @@ sub new {
     bless $self, __PACKAGE__;
     # print " new ";
     $self->{dtstart} = ref($class) ? $class->{dtstart} : undef;
+    $self->{wkst} =    ref($class) ? $class->{wkst}    : $WKST;
     return $self;
 }
 
@@ -171,7 +174,7 @@ you have a 'recurrence'.
 
 A recurrence is created by a 'recurrence rule':
 
-    $recurr = Date::Set->event( rule = 'FREQ=YEARLY' );   # all possible years
+    $recurr = Date::Set->event( rule => 'FREQ=YEARLY' );   # all possible years
 
 An unbounded recurrence like this cannot be printed. 
 It would take an infinitely long roll of paper.
@@ -532,9 +535,11 @@ sub exclude {
 
 =head2 wkst STRING
 
-    Date::Set::wkst('SU');
+    Date::Set::wkst('SU');    # global change
 
-Sets/reads the module's global "week start day". 
+    $set->wkst('MO');
+
+Sets/reads the "week start day". 
 
 The parameter must be one of 
 'MO' (default), 'TU', 'WE', 'TH', 'FR', 'SA', or 'SU'.
@@ -549,10 +554,18 @@ Return value is current wkst value.
 
 =cut
 
-# TODO: wkst is global - make it work locally too, if applied to an object
+# wkst works locally too, if applied to an object
 
 sub wkst {
-    my $value = pop;
+    my $class = shift;
+    my $value = shift;
+    $value = $class unless defined $value;
+    if (ref($class)) {
+        # carp " $class:",$class->{wkst}," => ", defined $value , ":$value ";
+        $class->{wkst} = uc($value) if (defined $value) and ($value =~ /^\w\w$/);
+        return $class->{wkst};
+    }
+    # carp " $class:$WKST => ", defined $value , ":$value ";
     $WKST = uc($value) if (defined $value) and ($value =~ /^\w\w$/);
     return $WKST;
 }
@@ -623,6 +636,12 @@ sub fevent {
 
     # carp "fevent $has";
 
+    # if ($has == 0) {
+    #    # start
+    #    # carp " SELF $self ".$self->new(-$inf, $inf);
+    #    return $self->new(-$inf, $inf) unless ref($self);
+    #   return $self;
+    # }
     if ($has == 1) {
         # start
         return $self->new($parm{start}, inf) unless ref($self);
@@ -986,8 +1005,12 @@ sub rrule {
 # This private routine parse an RRULE or an EXRULE and hands it back in the chunks
 # that recur_by_rule and exclude_by_rule expect
 sub _parse_rule {        
-    my ($rrule) = @_; 
-    my %return;
+    my $return = shift;
+    my ($rrule) = $$return{RRULE}; 
+
+    return unless defined $rrule;
+    # print "  RULE: [ $rrule ]\n";
+
     # RRULEs look like 'FREQ=foo;INTERVAL=bar;' etc.
      my @pieces = split(';', $rrule);
      foreach (@pieces) {
@@ -995,18 +1018,20 @@ sub _parse_rule {
        
         # BY<FOO> parameters should be arrays. everything else should be strings
          if ($name =~ /^BY/i) {
-            @{$return{$name}} = split(/,/,$value);
+            @{$$return{$name}} = split(/,/,$value);
          }
          else {
-            $return{$name} = $value;
+            $$return{$name} = $value;
          }
      }
-     return %return;
 }
 
 sub recur_by_rule {
     my $self = shift;
 
+    # $self = $self->numeric;
+    # carp "recur_by_rule $self";
+
     # TODO - put 'if' around the parsing so that 'backtrack' doesn't have
     # to redo it
 
@@ -1032,23 +1057,21 @@ sub recur_by_rule {
     my %parm = (
         # FREQ     => 'YEARLY',  # whatever
         INTERVAL => 1,
-        COUNT    => 999999,    # any big number
+        COUNT    => $inf, 
         UNTIL    => undef,
-        WKST     => $WKST,
+        # WKST     => $WKST,
         RRULE    => undef,
         PERIOD   => undef, 
         DTSTART  => undef,
         INCLUDE_DTSTART => 1,  # *ALWAYS* include DTSTART in result set
 		EXCLUDE_DTSTART => 0,  # DON'T remove DTSTART from result set
-        @parm
+        @parm,
     );
 
     my ( $rrule, %has );
-
-    # parse an RRULE out into its pieces.
-    if (defined $parm{'RRULE'} ) {
-        my %temp_parm = _parse_rule($parm{'RRULE'}); 
-        %parm = (%parm, %temp_parm);
+    if (exists $parm{'RRULE'}) {
+        _parse_rule(\%parm); # parse an RRULE out into its pieces.
+        undef $parm{'RRULE'};
     }
 
     # this is the constructor interface
@@ -1069,8 +1092,17 @@ sub recur_by_rule {
     # later! ---> $self->{dtstart} = $parm{DTSTART}   if defined $parm{DTSTART};
     # ... and vice-versa
 
-    $parm{DTSTART}   = $self->{dtstart} if defined $self->{dtstart};
-    $parm{DTSTART}   = $parm{PERIOD}->{dtstart} if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $parm{DTSTART};
+
+    $self = $self->numeric;
+    $parm{PERIOD} = $parm{PERIOD}->numeric if defined $parm{PERIOD};
+
+
+    $parm{DTSTART}   = $self->{dtstart} 
+        if defined $self->{dtstart} and not defined $parm{DTSTART};
+    $parm{DTSTART}   = $parm{PERIOD}->{dtstart} 
+        if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $parm{DTSTART};
+    $parm{DTSTART}   = $self->new( $parm{DTSTART} )->min 
+        if defined $parm{DTSTART};
     $parm{DTSTART_save} = $parm{DTSTART};   
 
     # print " dtstart is $parm{DTSTART} , was $self->{dtstart} \n";
@@ -1082,10 +1114,11 @@ sub recur_by_rule {
         $has{period} = 1;
         print " PERIOD: $parm{PERIOD} \n" if $DEBUG;
         # try to make $self smaller
+
+        # TODO: test for too_complex PERIOD? (we test $self later)
         $self = $self->intersection( $parm{PERIOD} );
 
-
-        # $self->{dtstart} = $parm{PERIOD}->{dtstart} if ref($parm{PERIOD}) and defined $parm{PERIOD}->{dtstart} and not defined $self->{dtstart};
+        print "  self intersected with period\n" if $DEBUG;
     }
     else {
         $has{period}  = 0;
@@ -1101,11 +1134,13 @@ sub recur_by_rule {
 
             # if we have "COUNT" we must start at DTSTART
             if ( defined $parm{DTSTART_save} ) {
-                if ( $parm{DTSTART_save} < $self->min ) {
+                # print "  ( $parm{DTSTART_save} < $self->min )  \n";
+                if ( ( $parm{DTSTART_save} < $self->min ) and 
+                     ($parm{COUNT} < $inf) ) {
                     $self = $self->union( $parm{DTSTART_save}, $self->min );
                 }
                 if ( $parm{DTSTART_save} > $self->min ) {
-                    $self = $self->intersection( $parm{DTSTART_save}, inf );
+                    $self = $self->intersection( $parm{DTSTART_save}, $inf );
                 }
             }
 
@@ -1118,6 +1153,7 @@ sub recur_by_rule {
     # carp " PERIOD $parm{PERIOD} ";
     # $DEBUG = 0;
 
+    print "  test until\n" if $DEBUG;
     # UNTIL and COUNT MUST NOT occur in the same 'recur'  (why?)
     if ( $parm{UNTIL} ) {
         # UNTIL
@@ -1128,10 +1164,12 @@ sub recur_by_rule {
     # this is the backtracking interface.
     # It allows the program to defer processing if it does not have enough
     # information to proceed.
+    # print "  testing self $self  min = ",$self->min," max = ",$self->max, "\n";
     if ( ( $self->{too_complex} )
-        or ( $self->min == -&inf )
-        or ( $self->max == &inf ) )
+        or ( $self->min and $self->min == -&inf )
+        or ( $self->max and $self->max == &inf ) )
     {
+
         my $b = $self->new();
         $self->trace( title => "rrule:backtrack" );
         print " BACKTRACKING \n" if $DEBUG;
@@ -1141,38 +1179,83 @@ sub recur_by_rule {
         $b->{parent}      = $self->copy;
         $b->{method}      = 'recur_by_rule';
         $b->{param}       = [%parm];
+
+        # TODO: set up {min} and {max}, if possible...
+
+
+        # now that we have a "function" we can try to find a valid subset 
+        #    if we have a COUNT
+        if (exists $parm{COUNT}  &&
+            exists $parm{FREQ}   &&
+            ($parm{COUNT} != &inf) &&
+            ($self->min   != -&inf)
+           ) {
+            # warn "we have COUNT=$parm{COUNT} INTERVAL=$parm{INTERVAL} and start=".$self->min." and FREQ=$parm{FREQ}";
+            my $count = $parm{COUNT} * $parm{INTERVAL};
+            my $try = $self->new( $self->min )->
+                offset( unit => $FREQ{$parm{FREQ}}, value => [0,$count] );
+            # warn "set = $try";
+            my $subset = $b->intersection($try);
+            # warn "subset = $subset";
+            my $size = 1 + $#{$subset->{list}};
+            if ( $size == $parm{COUNT} ) {
+                # warn "success: $size / $parm{COUNT}";
+                return $subset;
+            }
+            # TODO: if we were sure we never fail we wouldn't have to use backtracking
+            # warn "failed: $size / $parm{COUNT}";
+        }
+
+
+        # try to make a subset suitable for first() iteration
+        # warn "trying 'first': ";
+        if (exists $parm{FREQ}   &&
+            ($self->min   != -&inf)
+           ) {
+            # warn "we have COUNT=$parm{COUNT} INTERVAL=$parm{INTERVAL} and start=".$self->min." and FREQ=$parm{FREQ}";
+            my $count = $parm{INTERVAL};
+            my $try = $self->new( $self->min )->
+                offset( unit => $FREQ{$parm{FREQ}}, value => [0,$count] );
+            $try->{list}[0]{open_end} = 1;
+            # warn "\n    set = $try";
+            my $subset = $b->intersection($try);
+            # warn "\n    subset = $subset";
+            my $size = 1 + $#{$subset->{list}};
+            if ( $size > 0 ) {
+
+                # warn "\n    self: $self \n    parm: ".join(' ', map { "$_=$parm{$_}" } keys %parm );
+
+                # move $self ahead
+                $b->{parent} = $b->{parent}->complement($try);
+                @{$b->{min}} = $try->max_a;
+                # warn "    parent: ".$b->{parent};
+
+                return $subset->union($b);
+            }
+            else {
+                # warn "failed: 'first' $size";
+            }
+        }
+
+
         return $b;
     }
 
+    print "  self numeric\n" if $DEBUG;
+    $self = $self->numeric;   # again?
+    # $parm{PERIOD} = $parm{PERIOD}->numeric if $has{period};
+    # $parm{DTSTART} = $parm{DTSTART}->numeric if defined $parm{DTSTART};
+
     # -- don't do this before backtracking!
     $parm{PERIOD} = $self unless $has{period};
 
     my $when = $parm{PERIOD};
 
-    # NOTE: is this redundant? maybe already had it checked above...
-    # DTSTART gives the default values for month, day, h, m, s
-    unless ( defined $parm{DTSTART} ) {
-        $parm{DTSTART} = $parm{PERIOD}->min;
-    }
-    else {
-        # make sure DTSTART is the right type
-        $parm{DTSTART} = $self->new( $parm{DTSTART} )->min;
-
-        # apply DTSTART, just in case
-        $when = $when->intersection( $parm{DTSTART}, $FUTURE );
-    }
-
     # print " PARAMETERS: ", join(":", %parm), "\n";
 
     $when->_print( title => 'WHEN' ) if $DEBUG;
 
-#    # UNTIL and COUNT MUST NOT occur in the same 'recur'  (why?)
-#    if ( $parm{UNTIL} ) {
-#
-#        # UNTIL
-#        $when->_print( title => 'UNTIL' ) if $DEBUG;
-#        $when = $when->intersection( $PAST, $parm{UNTIL} );
-#    }
+    $parm{WKST} = $self->{wkst} unless defined $parm{WKST};
 
     if ( $parm{FREQ} ) {
 
@@ -1182,7 +1265,7 @@ sub recur_by_rule {
 
         $when->_print( title => 'FREQ' ) if $DEBUG;
 
-        if ( $self->max == &inf ) {
+        if ( $self->max == $inf ) {
 
             # TODO
             # that's real hard to quantize -- try to fix it
@@ -1198,6 +1281,7 @@ sub recur_by_rule {
 
         # -- WKST works here:   --> only if FREQ=WEEKLY; see also: BYWEEKNO
         if ( $parm{FREQ} eq 'WEEKLY' ) {
+
             my $wkst = $WEEKDAY{ $parm{WKST} };
 
             # print " [ wkst: $parm{WKST} = $wkst ] \n";
@@ -1227,13 +1311,14 @@ sub recur_by_rule {
         if ($parm{INTERVAL} > 1) {
             $freq = $freq
               # -- INTERVAL works here:
-              ->select( freq => $parm{INTERVAL}, count => 999999, strict => 0 )
-              ->_print(
+              ->select( freq => $parm{INTERVAL}, count => $inf, strict => 0 );
+            # carp "INTERVAL $parm{INTERVAL}";
+            $freq->_print(
                 title => 'FREQ('
                   . $parm{FREQ}
                   . ')+INTERVAL('
                   . $parm{INTERVAL} . ')'
-              )->compact;
+              )->compact if $DEBUG;
         }
 
         # -- BYSETPOS special handling -- 
@@ -1265,7 +1350,7 @@ sub recur_by_rule {
 
               # -- COUNT works here:
               ->select( freq => 1, count => $parm{COUNT}, strict => 0 )
-              ->_print( title => 'COUNT(' . $parm{COUNT} . ')' )
+              # ->_print( title => 'COUNT(' . $parm{COUNT} . ')' )
 
               # ->duration( unit => 'seconds', duration => 0 ) 
               ->offset(mode=>'begin', value=>[0,0])
@@ -1293,6 +1378,11 @@ sub recur_by_rule {
         $rrule = $rrule->complement( $parm{DTSTART_save} ) if $parm{EXCLUDE_DTSTART};
     }
 
+    # carp " returning $self ";
+    # carp "       and $rrule ";
+    # carp "        as " . $rrule->fixtype ;
+    # carp "       and " . $self->fixtype ;
+
     if ( $has{period} ) {
         return $self->union($rrule)->fixtype;
     }
@@ -1358,11 +1448,8 @@ sub _apply_DTSTART {
 
 # INTERNAL sub to calculate BYxxx
 #   input: $when, %parm
-
 #   output: $when (filtered)
 
-# TODO: This code needs to be refactored into ~ <50 line chunks and tested
-
 sub _rrule_by {
 
     print " [EVALUATING RRULE ",join(':',@_)," ]\n" if $DEBUG;
@@ -1408,7 +1495,7 @@ sub _rrule_by {
                     strict => 0,
                     fixtype => 0 )
                   # ->_print( title => $BYfoo . ":offset=" . $small_unit . join ( ',', @by ) )
-            )->no_cleanup;
+             )->no_cleanup;
             $when->_print( title => $BYfoo ) if $DEBUG;
             $has{months} = 1;
             $has{days}   = 1 if $small_unit eq 'days';
@@ -1517,13 +1604,14 @@ sub _rrule_by {
             $when = $when->intersection(
                 $when->quantize(
                     unit => $prev_unit,
+                    strict => 0,
                     fixtype => 0 )
                  ->offset(
                     mode   => 'circle',
                     unit   => $has,
                     value  => [@by],
                     strict => 0, fixtype => 0 )
-            )->no_cleanup;
+             )->no_cleanup;
             $when->_print( title => $BYx ) if $DEBUG;
             $has{$has} = 1;
         }
@@ -1535,13 +1623,16 @@ sub _rrule_by {
     if ( exists $parm{BYSETPOS} ) {
         my @by = @{ $parm{BYSETPOS} };
         foreach (@by) { $_-- if $_ > 0 }    # BY starts in 1; perl starts in 0
-        $when = $when->intersection( 
-           $when->compact
-           # ->_print(title=>'bysetpos1')
-           ->select( by => [@by] )
-           # ->_print(title=>'bysetpos2')
-        )->no_cleanup;
+        # $when = $when->intersection( 
+        #   $when->compact
+        #   # ->_print(title=>'bysetpos1')
+        #   ->select( by => [@by] )
+        #   # ->_print(title=>'bysetpos2')
+        # )->no_cleanup;
         $when->_print( title => 'BYSETPOS' ) if $DEBUG;
+
+        $when = $when->select( by => [@by] );
+        # carp " When: $when ";
     }
     # }}} 
 
@@ -1621,7 +1712,7 @@ sub exclude_by_rule {
     # print " [   period =     ", $parm{period}," ]\n";
     # print " [   rrule =      ", $parm{period}->recur_by_rule(%parm)," ]\n";
     # print " [   complement = ", $parm{period}->recur_by_rule(%parm)->complement," ]\n";
-    return $self->complement( $period->recur_by_rule(%parm) );
+    return $self->numeric->complement( $period->recur_by_rule(%parm) )->fixtype;
 }
 
 =head2 recur_by_date
@@ -1741,8 +1832,8 @@ sub occurrences {    # event->, period
         carp "$self -> occurrences called without a period argument";
     }
 
-    my $intersection =  $self->intersection( $parm{period} );
-    return($intersection);
+    my $intersection =  $self->numeric->intersection( $parm{period}->numeric );
+    return($intersection)->fixtype;
 }
 
 =head2 next_year, next_month, next_week, next_day, next_hour, next_minute, next_weekyear ($date_set)
@@ -1778,6 +1869,8 @@ See also previous note on 'weekyear' in 'About ISO 8601 week'.
 
 =cut
 
+sub DESTROY {}
+
 # try to make up for unwritten methods
 # currently defined are:
 #   next_XXX  (next_day, next_month ...), this_XXX,  prev_XXX
